// This demo shows how to create a shopping cart application using CGI-Expert
// and Pixelpack.
// If you are not familiar with the session variables of CGIExpert, this demo
// might be hard to understand, since it makes intensive use of these session
// variables. Consult the CGI-Expert manual for further information on session
// variables.
//
// written by Peter Strand, Femte Gear ( strand@femte-gear.dk )

unit shop;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  PxTemplate, PxCustomTable, PxTable, PxCustomText, PxText, HttpEng,
  GenEng, PxComponent, PxVisibleComponent, PxContainer, PxPage,
  PxPanel, HUtils, PxCustomImage, PxImage, PxOptimizer,
  PxCGIExpertOptimizer;

type
    // The TItem class describes entries in theDB

    TItem = class(tobject)
    public
          itemno : string;
          name   : string;
          desc   : string;
          price  : integer;
          Constructor CreateItem( NItemNo, NName, NDesc : string; NPrice : integer);
    end;

    // the TShopItem describes items in the shopping basket.
    // The itemindex refers to the zero based index of theDB
    // The qty (quantity) is not used in this demo
    TShopItem = class ( tobject )
    public
          ItemIndex : integer;
          qty       : integer;
          Constructor CreateItem( NItemIndex, NQty : integer);
    end;

    // The TCustomer class is used to make the customer data persistent.
    // It uses session variables to load and save its state.
    // The class holds information about what's in the shopping basket,
    // on what page the user is and where in theDB he is currently looking
    // Note that this makes a simple layer over the session variables in order
    // to avoid accessing the session variables in the actual code - might be
    // a little confusing, but it works great :-)
    TCustomer = class(TObject)
    public
          eng   : TGeneralHTTPEngine;
          items : tlist;
          TableIndex : integer;
          CurrentPage : string;
          Procedure LoadFromSession;
          Procedure SaveToSession;
          Constructor CreateCustomer(Neng : TGeneralHTTPEngine);
          Destructor destroy; override;
    end;


  TForm1 = class(TForm)
    PxPage1: TPxPage;
    engine: TGeneralHttpEngine;
    PxTemplate1: TPxTemplate;
    ItemList: TPxTable;
    CartPanel: TPxPanel;
    Shopcartlist: TPxTable;
    PxText4: TPxText;
    prevButton: TPxText;
    nextButton: TPxText;
    PxText2: TPxText;
    PxText3: TPxText;
    PxText1: TPxText;
    optimizer: TPxCGIExpertOptimizer;
    procedure PxTemplate1BeforePut(Sender: TObject);
    procedure engineExecRequest(Sender: TObject);
    procedure ShopcartlistbeforePut(Sender: TObject);
  
  private
    { Private declarations }
  public
    { Public declarations }
    theDB : Tlist;
    Customer : TCustomer;
  end;

  

var
  Form1: TForm1;

implementation

uses desc;
{$R *.DFM}

Constructor TShopItem.CreateItem( NItemIndex, NQty : integer);
begin
     inherited create;
     Qty:=NQty;
     Itemindex:=NItemIndex;
end;

Procedure TCustomer.LoadFromSession;
var
   t : TStringlist;
   i : integer;
   s : string;
begin

     TableIndex:=strtoint(sessionvar('ti','0'));
     currentpage:=sessionvar('cp','main');
     if currentpage='' then currentpage:='main';

     t:=SessionVarNames;
     for i:=0 to t.count-1 do
     begin
          s:=t[i];
          delete(s,3,length(s));
          if s='ii' then
             items.add(TShopItem.CreateItem(strtoint(sessionvar(t[i],'0')),1));
     end;
     
end;

Procedure TCustomer.SaveToSession;
var
   i : integer;
begin
     RemoveSessionvars;
     SetSessionVar('ti',inttostr(TableIndex));
     Setsessionvar('cp',currentpage);
     for i:=0 to items.count-1 do
        SetSessionVar('ii'+inttostr(i),inttostr(TShopItem(items[i]).itemIndex));

    

end;

Constructor TCustomer.CreateCustomer(Neng : TGeneralHTTPEngine);
begin
     inherited create;
     eng:=Neng;
     items:=tlist.create;
     tableindex:=0;
     CurrentPage:='';
end;

Destructor TCustomer.destroy;
var
   i : integer;
begin
     for i:=0 to items.count-1 do
         Tobject(items[i]).free;
     items.free;

     inherited destroy;
end;

Constructor TItem.CreateItem( NItemNo, NName, NDesc: string; NPrice : integer);
begin
    inherited create;
    itemno:=NItemno;
    name:= NName;
    desc:=ndesc;
    price:=NPrice;
end;

procedure TForm1.PxTemplate1BeforePut(Sender: TObject);
begin
     PxTemplate1.SetDisplayedComponent(cartPanel);
end;

procedure TForm1.engineExecRequest(Sender: TObject);
var
   s : string;
   n : integer;
   i : integer;
   t : Tstringlist;
begin
     // setup the demonstration "database"
     theDB := TList.create;
     theDB.add(TItem.CreateItem('34A','No-name Keyboard','',25));
     theDB.add(TItem.CreateItem('35' ,'Logitech mouse','',37));
     theDB.add(TItem.CreateItem('56A','CPU - fan ( AMD )','',11));
     theDB.add(TItem.CreateItem('56B','CPU - fan ( Intel )','',11));
     theDB.add(TItem.CreateItem('57' ,'220V Power Supply','',45));
     theDB.add(TItem.CreateItem('61' ,'Stereo Speakers','',32));
     theDB.add(TItem.CreateItem('61A','OKIJET Printer ZX500','',295));
     theDB.add(TItem.CreateItem('62' ,'ADI 17" monitor','',450));
     theDB.add(TItem.CreateItem('71' ,'ADI 15" monitor','',295));
     theDB.add(TItem.CreateItem('72' ,'CPU - Intel P2 233','',140));
     theDB.add(TItem.CreateItem('73' ,'CPU - Intel P2 266','',164));
     theDB.add(TItem.CreateItem('83' ,'CPU - Intel P2 300','',205));
     theDB.add(TItem.CreateItem('89' ,'CPU - Intel P2 350','',255));
     theDB.add(TItem.CreateItem('90' ,'CPU - Intel P2 400','',310));
     theDB.add(TItem.CreateItem('91' ,'Asus Motherboard ','',114));
     theDB.add(TItem.CreateItem('92' ,'Asus Motherboard ','',199));
     theDB.add(TItem.CreateItem('93' ,'RAM, 64 mb 100mzh ','',99));
     theDB.add(TItem.CreateItem('93A','RAM, 128 mb 100mzh ','',205));
     theDB.add(TItem.CreateItem('94' ,'No-name modem 33.6 ','',45));
     theDB.add(TItem.CreateItem('94A','US Robotics ISDN 128','',123));
     theDB.add(TItem.CreateItem('95' ,'Coax 10 ft.','',9));
     theDB.add(TItem.CreateItem('96' ,'Coax 100 ft.','',50));
     theDB.add(TItem.CreateItem('97' ,'Coax 500 ft.','',200));
     theDB.add(TItem.CreateItem('98 ','Microline Scanner A4','',95));
     theDB.add(TItem.CreateItem('99' ,'HP A4 Scanner','',135));


     //  start the session and load the customer from the session variables
     StartSession(Now+1/24);
     Customer := TCustomer.CreateCustomer(engine);
     Customer.LoadFromSession;

   //  check for form parameters
   //  ******************************************************

     // adds an item to the shopping cart
     s := formvar('additem','');
     if s <> '' then
     begin
          n:=strtoint(s);
          customer.items.add(TShopItem.createItem(n,1));
     end;

     // remove an item from the shopping cart
     s := formvar('removeitem','');
     if s <> '' then
     begin
          n:=strtoint(s);
          if customer.items.count>n then
             customer.items.delete(n);
     end;

     // set the position in the product DB ( controlled by next / prev buttons
     s := formvar('settableindex','');
     if s <> '' then
     begin
          n:=strtoint(s);
          customer.tableIndex:=n;
     end;

     // set what page should be displayed ( main / product detail / order etc )
     s := formvar('setcurrentpage','');
     if s <> '' then
        customer.currentpage:=s;


     // OK , done with customer data. We can save it now.
     Customer.SaveToSession;
     if Customer.CurrentPage='main' then
     begin

          itemlist.startIndex:=Customer.TableIndex;
          for i:=0 to theDB.count-1 do
          begin
               ItemList.Items[0,i].text:=TItem(theDB[i]).itemNo;
               ItemList.Items[1,i].text:=TItem(theDB[i]).name;
               ItemList.Items[1,i].link:='webshop.exe?setcurrentpage=productinfo'+inttostr(i);
               ItemList.Items[2,i].text:=inttostr(TItem(theDB[i]).price)+'.00';
               ItemList.Items[3,i].text:='order';
               ItemList.Items[3,i].link:='webshop.exe?additem='+inttostr(i);
          end;

          nextbutton.link:='';
          prevbutton.link:='';

          if itemlist.numberOfRows>(itemlist.StartIndex+itemlist.MaxRows)  then
          begin
               i:=itemlist.StartIndex+itemlist.MaxRows;
               nextbutton.link:='webshop.exe?settableindex='+inttostr(i);
          end;

          if itemlist.StartIndex>0 then
          begin
               i:=itemlist.StartIndex-itemlist.MaxRows;
               if i<0 then i:=0;
               prevbutton.link:='webshop.exe?settableindex='+inttostr(i);

          end;
          log('put form1.page1');
          put(form1.pxPage1.put(optimizer));
          log('did put form1.page1');          
     end
     else

     if pos('productinfo',Customer.CurrentPage)<>0 then
     begin
        s:=Customer.CurrentPage;
        delete(s,1,11);
        form2.prodnametext.text.clear;
        form2.prodnametext.text.add(TItem(theDB[strtoint(s)]).name);
        form2.pricetext.text.clear;
        form2.pricetext.text.add('$'+inttostr(TItem(theDB[strtoint(s)]).price)+'.00');
        form2.Ordertext.link:='webshop.exe?additem='+s;
        put(form2.pxPage1.put(optimizer));

     end;
     log('were done');
     Customer.free;
     theDB.free;
end;

procedure TForm1.ShopcartlistbeforePut(Sender: TObject);
var
   i : integer;
   tot : integer;
   rows: integer;

begin
     tot:=0;
     rows:=0;
     Shopcartlist.EmptyTable;
     for i := 0 to customer.items.count-1 do
     begin
          if rows < 10 then
          begin
               ShopCartlist.items[0,i].link:='webshop.exe?removeitem='+inttostr(i);
               ShopCartlist.items[0,i].text:=TItem(theDB.items[TShopItem(customer.items[i]).itemIndex]).name;
               ShopCartlist.items[1,i].text:=inttostr(TItem(theDB.items[TShopItem(customer.items[i]).itemIndex]).price)+'.00';
               inc(rows);
          end
          else
               Shopcartlist.items[0,10].text:='(more)';

          tot:=tot+TItem(theDB.items[TShopItem(customer.items[i]).itemIndex]).price;

     end;
     Shopcartlist.items[0,rows+1].text:='TOTAL';
     Shopcartlist.items[1,rows+1].text:=inttostr(tot)+'.00';
end;

end.
 