unit webmessages;

interface
uses classes,PxTable;
type

    TMessageDesc = record
     number      : integer;
     parent      : integer;
     date        : TDateTime;
     header      : string[200];
     sender      : string[200];
     replymail   : string[200];
     mesFile     : string[200];
    end;

    TForumMessage = class;

    TMessageManager = class(TObject)
    private
          DataFile  : file of TMessageDesc;
          Cur_msg   : integer;
          m_Messages  : Tlist;
          Function  GetMessageByIndex(i : integer) : TForumMessage;
          Function  GetMessageByNumber(i : integer) : TForumMessage;
          Procedure LoadMessagesToCache;
    public
          LocalPath : string;
          forumName : string;
          table     : TPxTable;
          lastRow   : integer;
          index : integer;
          Function messageCount : integer;
          constructor Create;
          destructor Destroy;
          function  FindMessageInFile( messagenumber : integer) : integer;
          procedure AddMessage(m : TMessageDesc; txt : string);
          procedure AddMessageToTable(t : TForumMessage; i : integer; killable : boolean);
          procedure DeleteMessage(messagenumber : integer);
          function  GetUniqueMessageNumber : integer;
          function  initialize :  boolean;
          property  MessagesByNumber[index : integer] : TForumMessage read GetMessageByNumber;
          property  MessagesByIndex[index : integer] : TForumMessage read GetMessageByIndex;

    end;

    TForumMessage = class(TObject)
    private
          m_Text    : TStringlist;
          m_Manager : TMessageManager;
          function GetText :  TStringlist;
    public
          Desc : TMessageDesc;
          property text : TStringlist read GetText;
          Constructor CreateMessage(m : TMessageDesc; parent : TMessageManager);
    end;

implementation
uses sysutils;

function  TMessageManager.FindMessageInFile( messagenumber : integer) : integer;
var
   i : integer;
   m : TMessageDesc;
begin
     i:=-1;
     result:=-1;
     seek(datafile,0);
     repeat
           if EOF(datafile) then
           begin
              i:=-1;
              break;
           end;
           inc(i);
           seek(datafile,i);
           read(datafile,m);
     until m.number=messagenumber;

     result:=i;

end;

procedure TMessageManager.DeleteMessage(messagenumber : integer);
var
   i : integer;
   m : TMessageDesc;
begin
     i:=findmessageInFile(messagenumber);
     if i>-1 then
     begin

          m.number:=-1;
          m.parent:=-1;
          m.date:=-1;
          m.header:='';
          m.sender:='';
          m.replymail:='';
          m.mesFile:='';

          seek(datafile,i);
          write(datafile,m);

          i:=0;
          seek(datafile,0);
          while not EOF(datafile) do
          begin
               seek(datafile,i);
               read(datafile,m);
               if m.parent=messagenumber
               then deleteMessage(m.number);
               inc(i);
          end;
     end;
end;

procedure TMessageManager.AddMessageToTable(t : TForumMessage; i : integer;killable : boolean);
var
   s : Integer;

   space : string;
begin
     space:='';
     for s:=0 to i-1 do
         space:=space+'&nbsp&nbsp&nbsp';

     Table.items[0,lastRow].text:=space+'<A HREF="forum.exe?nm='+forumName+'&index='+inttostr(index)+'&view='+inttostr(t.desc.number)+'">'+t.desc.Header+'</A>';
     if killable then Table.items[0,lastRow].text:=Table.items[0,lastRow].text+'<A HREF="forum.exe?nm='+forumName+'&index='+inttostr(index)+'&kill='+inttostr(t.desc.number)+'">&nbsp&nbsp&nbsp (delete) </A>';
     Table.items[2,lastRow].text:=DateTimeToStr(t.desc.date);
     Table.items[1,lastRow].text:=t.desc.sender;
     Table.items[1,lastRow].link:='MAILTO:'+t.desc.replymail;
     inc(lastRow);
     for s:=0 to m_Messages.count-1 do
     begin
          if ((MessagesByIndex[s].desc.parent=t.desc.number) and ( t.desc.number>-1))
             then AddMessageToTable(messagesByIndex[s],i+1,killable);
     end;

end;

Function TMessageManager.messageCount : integer;
begin
     if m_Messages=nil then
        LoadMessagesToCache;

     result:=m_Messages.count;

end;

Procedure TMessageManager.LoadMessagesToCache;
          Function SortMessages( item1,item2 : pointer) : integer;
          var
             r : real;
          begin
               r:=(TForumMessage(item2).desc.date)-(TForumMessage(item1).desc.date);
               if r=0.0 then result:=0;
               if r<0 then result:=-1;
               if r>0 then result:=1;
          end;

var
   m : TForumMessage;
   r : TMessageDesc;
begin
     seek(datafile,0);
     m_Messages:=TList.create;
     while not EOF(DataFile) do
     begin
          read(datafile,r);
          m:=TForumMessage.CreateMessage(r,self);
          m_Messages.add(m);
     end;

     m_Messages.sort(@sortmessages);
end;

Constructor TMessageManager.Create;
begin
     inherited create;
     lastRow:=0;
end;

destructor TMessageManager.Destroy;
begin
     m_Messages.free;
     inherited Destroy;
end;

Constructor TForumMessage.CreateMessage(m : TMessageDesc; parent : TMessageManager);
begin
     inherited create;
     m_Manager := parent;
     desc:=m;
end;

function TForumMessage.GetText : TStringlist;
begin
     if m_text=nil then
     begin
          m_Text:=TStringlist.create;
          m_Text.loadFromFile(desc.MesFile);
     end;

     result:=m_Text;
end;

function TMessageManager.initialize : boolean;
var
   fname : string;
begin
    Cur_Msg := 0;
    fname := localpath+'\'+forumname+'_data.bin';
    assignFile(Datafile,fname);
    if not fileExists(fname) then
       rewrite(datafile);

    reset(datafile);

end;


Function TMessageManager.GetUniqueMessageNumber : integer;
var
   max : longint;
   T : TMessageDesc;
begin
   reset(datafile);
   max:=0;
   while not(eof(datafile)) do
   begin
        read(datafile,t);
        if t.number>=max then max:=t.number;
   end;

   result:=max+1;
end;

Function TMessageManager.GetMessageByNumber(i : integer) : TForumMessage;
var
   s : integer;
   t : TForumMessage;
begin
     if m_Messages=nil then
          LoadMessagesToCache;

     for s:=0 to m_Messages.count-1 do
     begin
         if TForumMessage(m_Messages[s]).desc.number=i then
         begin
              result:=m_Messages[s];
              break;
         end;
     end;
end;

Function TMessageManager.GetMessageByIndex(i : integer) : TForumMessage;
var
   s : integer;
   t : TForumMessage;
begin
     if m_Messages=nil then
          LoadMessagesToCache;

     result:=m_Messages[i];
end;



procedure TMessageManager.addMessage(m : TMessageDesc; txt : string);
var
   t : TStringlist;
   i : integer;
   p : TMessageDesc;
begin
     i:=0;

     while not (eof(datafile)) do
     begin
           seek(datafile,i);
           read(datafile,p);
           inc(i);
     end;
     if not (eof(datafile)) then
        seek(datafile,i);
        
     write(datafile,m);
     t:=TStringlist.create;
     t.add(txt);
     t.savetofile(m.mesfile);
     t.free;
end;


end.
